import urandom as random

class GameBoard:
    def __init__(self):
        self.field = [
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]
        ]
    
    def clear(self):
        self.field = [
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]
        ]

    def put(self, num, x, y):
        if self.field[y][x] != 0:
            return False
        self.field[y][x] = num
        return True

    # returns [-1to1, [y,x],[y,x],[y,x]]
    # -1 or 1: bingo with ai or player
    # 0: still no bingo
    def check_bingo(self):
        for y in range(3):
            if self.field[y][0] == self.field[y][1] == self.field[y][2] and self.field[y][2] != 0:
                return [self.field[y][0], [y, 0], [y, 1], [y,2]]
        for x in range(3):
            if self.field[0][x] == self.field[1][x] == self.field[2][x] and self.field[2][x] != 0:
                return [self.field[0][x], [0, x], [1, x], [2, x]]
        if self.field[0][0] == self.field[1][1] == self.field[2][2] and self.field[2][2] != 0:
            return [self.field[0][0], [0, 0], [1, 1], [2, 2]]
        if self.field[0][2] == self.field[1][1] == self.field[2][0] and self.field[2][0] != 0:
            return [self.field[0][2], [0, 2], [1, 1], [2, 0]]
        return [0, [9,9], [9,9], [9,9]]

    def get_cell_count(self, num):
        cells = 0
        for row in self.field:
            for cell in row:
                if cell == num:
                    cells += 1
        return cells

    def put_random(self):
        empty_cell_count = self.get_cell_count(0)
        rand_num = random.getrandbits(4) # 0 - 15
        while rand_num > empty_cell_count - 1:
            rand_num = random.getrandbits(4)

        cnt = 0
        for y in range(3):
            for x in range(3):
                if self.field[y][x] == 0:
                    if rand_num == cnt:
                        self.field[y][x] = -1
                        return [y, x]
                    cnt += 1

    # The cell one more to win
    def will_win(self, num):
        for y in range(3):
            if self.field[y][0] + self.field[y][1] + self.field[y][2] == num * 2:
                for x in range(3):
                    if self.field[y][x] == 0:
                        return [y, x]
        for x in range(3):
            if self.field[0][x] + self.field[1][x] + self.field[2][x] == num * 2:
                for y in range(3):
                    if self.field[y][x] == 0:
                        return [y, x]
        if self.field[0][0] + self.field[1][1] + self.field[2][2] == num * 2:
            if self.field[0][0] == 0:
                return [0, 0]
            if self.field[1][1] == 0:
                return [1, 1]
            return [2, 2]
        if self.field[0][2] + self.field[1][1] + self.field[2][0] == num * 2:
            if self.field[0][2] == 0:
                return [0, 2]
            if self.field[1][1] == 0:
                return [1, 1]
            return [2, 0]
        return None

 
    def put_auto(self):
        ai_will_win_cell = self.will_win(-1)
        if ai_will_win_cell != None:
            self.field[ai_will_win_cell[0]][ai_will_win_cell[1]] = -1
            return [ai_will_win_cell[0], ai_will_win_cell[1]]
            
        players_will_win_cell = self.will_win(1)
        if players_will_win_cell != None:
            self.field[players_will_win_cell[0]][players_will_win_cell[1]] = -1
            return [players_will_win_cell[0], players_will_win_cell[1]]

        return self.put_random()
        
        


