# Resolution=Low: 110x88, 16 colors

from frame_manager import FrameManager
from input_manager import InputManager
from scene_game import SceneGame
from scene_title import SceneTitle
import upygame as upg


SCREEN_WIDTH = 110
SCREEN_HEIGHT = 88

SCENE_TITLE = 1
SCENE_GAME = 2

MAX_FPS = 20

class MyGame:

    def __init__(self):
        self.screen = upg.display.set_mode()
        self.input_manager = InputManager()
        self.frame_manager = FrameManager()
        self.requested_next_scene = None
        self.current_scene = self.create_scene(SCENE_TITLE)
        upg.display.set_palette_16bit([
            0xf81f, 0x3165, 0xa2a0, 0xed07, 0xfe31, 0xeed3, 0x2944, 0x59a1, 0x946f, 0x2944, 0xf987, 0xbb42
        ])
        upg.draw.set_transparent_color(0)



    def main(self):
        while True:
            self.frame_manager.update_frame_count()
            self.input_manager.check_key_input()
            self.current_scene.execute()        
            upg.display.flip()
            self.change_scene_if_requested()


    def request_change_scene(self, next_scene):
        self.requested_next_scene = next_scene

    def create_scene(self, next_scene):
        if next_scene == SCENE_TITLE:
            return SceneTitle(self, self.input_manager, self.frame_manager)
        elif next_scene == SCENE_GAME:
            return SceneGame(self, self.input_manager, self.frame_manager)
        else:
            return None

    def change_scene_if_requested(self):
        if self.requested_next_scene == None:
            return
        self.current_scene = self.create_scene(self.requested_next_scene)
        self.requested_next_scene = None
        

game = MyGame()
game.main()
