
from abstract_scene import AbstractScene
from frame_manager import FrameManager
from game_board import GameBoard
from input_manager import InputManager
import sprite_data as mysprite
import upygame as upg
import urandom as random

class SceneGame(AbstractScene):

    PHASE_FIELD_DRAWING = 0
    PHASE_PLAYER_TURN = 1
    PHASE_WRITING_CIRCLE = 2
    PHASE_AI_TURN = 3
    PHASE_WRITING_CROSS = 4
    PHASE_RESULT = 5
    PHASE_RETRY_DIALOG = 6
    PHASE_CHECK_BINGO = 7

    TURN_PLAYER = 0
    TURN_AI = 1

    def __init__(self, my_game, input_manager, frame_manager):
        # assert isinstance(my_game, MyGame)
        assert isinstance(input_manager, InputManager)
        assert isinstance(frame_manager, FrameManager)
        self.initialize(my_game, input_manager, frame_manager)


    def initialize(self, my_game, input_manager, frame_manager):
        self.my_game = my_game
        self.input_manager = input_manager
        self.frame_manager = frame_manager
        self.first_frame = self.frame_manager.total_frame
        self.cursor_x = 0
        self.cursor_y = 0
        self.turn = random.getrandbits(1) # 0:player, 1:ai
        self.phase = self.PHASE_FIELD_DRAWING
        self.board = GameBoard()
        self.cell_put_frame = 0
        self.is_initialized = False
        self.bingo_result = None


    def draw_field(self):
        from main import SCREEN_HEIGHT, SCREEN_WIDTH
        upg.draw.rect(upg.Rect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT), True, 2)
        y = 0
        while y < SCREEN_HEIGHT:
            y += 8
            x = 0
            for times in range(10):
                x += 10
                upg.draw.pixel(x, y, 11)

        
        self.my_game.screen.blit(mysprite.sf_paper, 20, 10)
        
        box_left = 34
        box_top = 33
        line_space = 15
        
        frames_in_this_scene = self.frame_manager.total_frame - self.first_frame
        # horizontal 1
        if frames_in_this_scene <= 10:
            upg.draw.line(
                box_left,
                box_top + line_space,
                box_left + line_space * 3 * frames_in_this_scene // 10,
                box_top + line_space,
                6)
            self.my_game.screen.blit(mysprite.sf_pen, box_left + line_space * 3 * frames_in_this_scene // 10 - 3, box_top + line_space - 30)
        else:
            upg.draw.line(box_left, box_top + line_space, box_left + line_space * 3, box_top + line_space, 6)
            
        # horizontal 2
        if frames_in_this_scene >= 11 and frames_in_this_scene <= 20:
            upg.draw.line(
                box_left,
                box_top + line_space * 2,
                box_left + line_space * 3 * (frames_in_this_scene - 10) // 10,
                box_top + line_space * 2,
                6)
            self.my_game.screen.blit(mysprite.sf_pen, box_left + line_space * 3 * (frames_in_this_scene - 10) // 10 - 3, box_top + line_space * 2 - 30)
        elif frames_in_this_scene > 20:
            upg.draw.line(
                box_left,
                box_top + line_space * 2,
                box_left + line_space * 3,
                box_top + line_space * 2,
                6)
        
        # vertical 1
        if frames_in_this_scene >= 21 and frames_in_this_scene <= 30:
            upg.draw.line(
                box_left + line_space,
                box_top,
                box_left + line_space,
                box_top + line_space * 3 * (frames_in_this_scene - 20) // 10,
                6)
            self.my_game.screen.blit(mysprite.sf_pen, box_left + line_space - 3, box_top + line_space * 3 * (frames_in_this_scene - 20) // 10 - 30)
        elif frames_in_this_scene > 30:
            upg.draw.line(
                box_left + line_space,
                box_top,
                box_left + line_space,
                box_top + line_space * 3,
                6)

        # vertical 2
        if frames_in_this_scene >= 31 and frames_in_this_scene <= 40:
            upg.draw.line(
                box_left + line_space * 2,
                box_top,
                box_left + line_space * 2,
                box_top + line_space * 3 * (frames_in_this_scene - 30) // 10,
                6)
            self.my_game.screen.blit(mysprite.sf_pen, box_left + line_space * 2 - 3, box_top + line_space * 3 * (frames_in_this_scene - 30) // 10 - 30)
        elif frames_in_this_scene > 40:
            upg.draw.line(
                box_left + line_space * 2,
                box_top,
                box_left + line_space * 2,
                box_top + line_space * 3,
                6)
            if not self.is_initialized:
                if self.turn == self.TURN_PLAYER:
                    self.phase = self.PHASE_PLAYER_TURN
                else:
                    self.phase = self.PHASE_AI_TURN
                self.is_initialized = True

    def change_turn(self):
        if self.turn == self.TURN_PLAYER:
            self.turn = self.TURN_AI
        else:
            self.turn = self.TURN_PLAYER

    def draw_cursor(self):
        box_left = 34
        box_top = 33
        line_space = 15
        rect = upg.Rect(
            box_left + self.cursor_x * line_space + 1,
            box_top + self.cursor_y * line_space + 1,
            line_space - 1,
            line_space - 2)
        upg.draw.rect(rect, True, 4)

        if self.phase == self.PHASE_RESULT:
            upg.draw.rect(upg.Rect(box_left + self.bingo_result[1][1] * line_space + 1, box_top + self.bingo_result[1][0] * line_space + 1, line_space - 1, line_space - 2), True, 4)
            upg.draw.rect(upg.Rect(box_left + self.bingo_result[2][1] * line_space + 1, box_top + self.bingo_result[2][0] * line_space + 1, line_space - 1, line_space - 2), True, 4)
            upg.draw.rect(upg.Rect(box_left + self.bingo_result[3][1] * line_space + 1, box_top + self.bingo_result[3][0] * line_space + 1, line_space - 1, line_space - 2), True, 4)

        if self.phase == self.PHASE_WRITING_CIRCLE or self.phase == self.PHASE_WRITING_CROSS or self.board.field[self.cursor_y][self.cursor_x] == 0:
            if self.frame_manager.current_frame_in_sec > 10:
                sprite = mysprite.sf_gray_cross if self.phase == self.PHASE_WRITING_CROSS else mysprite.sf_gray_circle
                self.my_game.screen.blit(sprite, box_left + self.cursor_x * line_space + 1, box_top + self.cursor_y * line_space + 1)


    def draw_marks(self):

        box_left = 34
        box_top = 33
        line_space = 15
        # circle/cross
        for y in range(3):
            for x in range(3):
                # The mark placed just before is displayed in gray, so it is not drawn here.
                if self.phase == self.PHASE_WRITING_CIRCLE or self.phase == self.PHASE_WRITING_CROSS:
                    if self.cursor_y == y and self.cursor_x == x:
                        continue
                if self.board.field[y][x] == 1:
                    self.my_game.screen.blit(mysprite.sf_circle, box_left + x * line_space + 1, box_top + y * line_space + 1)
                elif self.board.field[y][x] == -1:
                    self.my_game.screen.blit(mysprite.sf_cross, box_left + x * line_space + 1, box_top + y * line_space + 1)
        # pen
        frames_after_put = self.frame_manager.total_frame - self.cell_put_frame

        if self.phase == self.PHASE_WRITING_CIRCLE or self.phase == self.PHASE_WRITING_CROSS:
            if frames_after_put > 0 and frames_after_put <= 7:
                start_x = box_left + self.cursor_x * line_space + 4
                start_y = box_top + self.cursor_y * line_space + 4
                self.my_game.screen.blit(mysprite.sf_pen, start_x - (frames_after_put // 2) - 5, start_y + frames_after_put - 30)
            elif frames_after_put >= 8 and frames_after_put <= 14:
                start_x = (box_left + self.cursor_x * line_space + 4) - (7 // 2)
                start_y = (box_top + self.cursor_y * line_space + 4) + 7
                self.my_game.screen.blit(mysprite.sf_pen, start_x + frames_after_put - 5, start_y - frames_after_put - 30)
            elif frames_after_put >= 15 and frames_after_put <= 22:
                start_x = (box_left + self.cursor_x * line_space + 4) - (7 // 2) + 14
                start_y = (box_top + self.cursor_y * line_space + 4) + 7 - 14
                self.my_game.screen.blit(mysprite.sf_pen, start_x - (frames_after_put // 2) - 5, start_y + frames_after_put - 30)
            else:
                # only once
                if frames_after_put == 23:
                    self.phase = self.PHASE_CHECK_BINGO


    def input_key(self):
        from main import SCENE_TITLE

        if self.phase == self.PHASE_PLAYER_TURN:
            if self.input_manager.first_pressed_key_in_current_frame() == upg.K_RIGHT:
                self.cursor_x = self.cursor_x + 1 if self.cursor_x < 2 else 0
            elif self.input_manager.first_pressed_key_in_current_frame() == upg.K_LEFT:
                self.cursor_x = self.cursor_x - 1 if self.cursor_x > 0 else 2
            elif self.input_manager.first_pressed_key_in_current_frame() == upg.K_DOWN:
                self.cursor_y = self.cursor_y + 1 if self.cursor_y < 2 else 0
            elif self.input_manager.first_pressed_key_in_current_frame() == upg.K_UP:
                self.cursor_y = self.cursor_y - 1 if self.cursor_y > 0 else 2

            elif self.input_manager.first_pressed_key_in_current_frame() == upg.BUT_A:
                put_succeeded = self.board.put(1, self.cursor_x, self.cursor_y)
                if put_succeeded:
                    self.cell_put_frame = self.frame_manager.total_frame
                    self.phase = self.PHASE_WRITING_CIRCLE
        
        elif self.phase == self.PHASE_RESULT:
            if self.input_manager.first_pressed_key_in_current_frame() == upg.BUT_A:
                self.initialize(self.my_game, self.input_manager, self.frame_manager)
            elif self.input_manager.first_pressed_key_in_current_frame() == upg.BUT_B:
                self.my_game.request_change_scene(SCENE_TITLE)

    def draw_dialog(self):
        if self.bingo_result[0] == 1:
            if self.frame_manager.current_frame_in_sec > 10:
                self.my_game.screen.blit(mysprite.sf_you_win, 10, 1)
        elif self.bingo_result[0] == -1:
            if self.frame_manager.current_frame_in_sec > 10:
                self.my_game.screen.blit(mysprite.sf_you_lose, 10, 3)
        upg.draw.rect(upg.Rect(1, 70, 109, 15), True, 7)
        upg.draw.text(1, 74, "[A] RETRY [B] TITLE", 3)

        

    def execute(self):

        self.input_key()
        self.draw_field()


        # ai
        if self.phase == self.PHASE_AI_TURN:
            cursor = self.board.put_auto()
            self.cursor_x = cursor[1]
            self.cursor_y = cursor[0]
            self.cell_put_frame = self.frame_manager.total_frame
            self.phase = self.PHASE_WRITING_CROSS


        if self.phase == self.PHASE_PLAYER_TURN or self.phase == self.PHASE_WRITING_CIRCLE:
            upg.draw.text(5, 3, "YOUR TURN", 5)
        if (self.phase == self.PHASE_PLAYER_TURN
            or self.phase == self.PHASE_WRITING_CIRCLE
            or self.phase == self.PHASE_AI_TURN
            or self.phase == self.PHASE_WRITING_CROSS 
            or self.phase == self.PHASE_RESULT):

            self.draw_cursor()

        self.draw_marks()
        
        if self.phase == self.PHASE_CHECK_BINGO:
            self.bingo_result = self.board.check_bingo()
            if self.bingo_result[0] == 0:
                # check draw
                if self.board.get_cell_count(0) == 0:
                    self.board.clear()
                else:
                    self.change_turn()
                    self.phase = self.PHASE_PLAYER_TURN if self.turn == self.TURN_PLAYER else self.PHASE_AI_TURN
            else:
                self.phase = self.PHASE_RESULT

        if self.phase == self.PHASE_RESULT:
            self.draw_dialog()

        # upg.draw.text(1, 10, str(random.getrandbits(1)), 3)

